------------------------------------------------------- 
-- A Video Frame Grabber. 
-- 
-- This circuit was first described in "Practical Design 
-- Using Programmable Logic" by David Pellerin and Michael 
-- Holley (Prentice Hall, 1990). A slightly modified form 
-- of the circuit also appears in the ATMEL Configurable 
-- Logic Design and Application Book, 1993-1994 edition. 
-- 
-- The circuit described is a simple freeze-frame unit that 
-- 'grabs' and holds a single frame of NTSC color video 
-- image. This design description includes the frame detection 
-- and capture logic. The complete circuit requires an 8-bit 
-- D-A/A-D converter and a 256K X 8 static RAM. 
-- 
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity CONTROL is
  port (
    rst      : in  std_logic;
    clk      : in  std_logic;
    mode     : in  std_logic;
    data     : in  std_logic_vector(7 downto 0);
    TestLoad : in  std_logic;
    addr     : out std_logic_vector(17 downto 0);
    RAMWE    : out std_logic;
    RAMOE    : out std_logic;
    ADOE     : out std_logic
  ); 
end CONTROL;

architecture behavioral of CONTROL is
  
  constant FRAMESIZE : integer := 253243;
  constant TESTADDR  : integer := 253000;

  signal ENDFR : std_logic;
  signal INCAD : std_logic;
  signal vs    : std_logic;
  signal sync  : unsigned (7 downto 0);
  
  type   states is (StateLive, StateWait, StateSample, StateDisplay);
  signal current_state, next_state : states;
  
begin
  -- Address counter. This counter increments until we reach the end of 
  -- the frame (address 253243), or until the input INCAD goes low. 
  ADDRCTR : process(clk)
    
    variable cnt : unsigned(17 downto 0);
    
  begin
    if rising_edge(clk) then
      if (TestLoad = '1') then
        cnt   := to_unsigned(TESTADDR, 18);
        ENDFR <= '0';
      else
        if (INCAD = '0' or cnt = to_unsigned(FRAMESIZE, 18)) then
          cnt := (others => '0');
        else
          cnt := cnt + 1;
        end if;
        if (cnt = FRAMESIZE) then
          ENDFR <= '1';
        else
          ENDFR <= '0';
        end if;
      end if;
    end if;
    addr <= std_logic_vector(cnt);
  end process;

  -- Vertical sync detector. Here we look for 128 bits of zero, which 
  -- indicates the vertical sync blanking interval. 
  SYNCCTR : process(rst, clk)
  begin
    if (rst = '1') then
      sync <= (others => '0');
    elsif rising_edge(clk) then
      if (data /= "00000000" or sync = to_unsigned(127, 7)) then
        sync <= (others => '0');
      else
        sync <= sync + 1;
      end if;
    end if;
  end process;

  vs <= '1' when sync = to_unsigned(127, 7) else '0';

  -- State register process: 
  STREG : process(rst, clk)
  begin
    if (rst = '1') then
      current_state <= StateLive;
    elsif rising_edge(clk) then
      current_state <= next_state;
    end if;
  end process;

  -- State transitions: 
  STTRANS : process(current_state, mode, vs, ENDFR)
  begin
    case current_state is
      when StateLive =>                 -- Display live video on the output 
        RAMWE <= '1';
        RAMOE <= '1';
        ADOE  <= '0';
        INCAD <= '0';
        if (mode = '1') then
          next_state <= StateWait;
        end if;
        
      when StateWait =>                 -- Wait for vertical sync 
        RAMWE <= '1';
        RAMOE <= '1';
        ADOE  <= '0';
        INCAD <= '0';
        if (vs = '1') then
          next_state <= StateSample;
        end if;
        
      when StateSample =>               -- Sample one frame of video 
        RAMWE <= '0';
        RAMOE <= '1';
        ADOE  <= '0';
        INCAD <= '1';
        if (ENDFR = '1') then
          next_state <= StateDisplay;
        end if;
        
      when StateDisplay =>              -- Display the stored frame 
        RAMWE <= '1';
        RAMOE <= '0';
        ADOE  <= '1';
        INCAD <= '1';
        if (mode = '1') then
          next_state <= StateLive;
        end if;
    end case;
  end process;
  
end behavioral;
